From 466f6cb994babf2edf3b3e496baafa1a70958415 Mon Sep 17 00:00:00 2001
From: Pierre d'Herbemont <pdherbemont@videolan.org>
Date: Fri, 16 May 2008 14:42:39 +0200
Subject: [PATCH] objects: Protect from FIND_PARENT directly in the FIND_PARENT cases.

This allows a correct vlc_object_detach(p_child) behaviour. Because vlc_object_detach() won't run if p_parent is set to NULL.
---
 src/misc/objects.c |   25 ++++++++++++++++++++-----
 1 files changed, 20 insertions(+), 5 deletions(-)

diff --git a/src/misc/objects.c b/src/misc/objects.c
index 4c6547d..12da0bd 100644
--- a/src/misc/objects.c
+++ b/src/misc/objects.c
@@ -879,9 +879,8 @@ void __vlc_object_release( vlc_object_t *p_this )
         /* Detach from parent to protect against FIND_CHILDREN */
         if (p_this->p_parent)
             vlc_object_detach_unlocked (p_this);
-        /* Detach from children to protect against FIND_PARENT */
-        for (int i = 0; i < p_this->i_children; i++)
-            p_this->pp_children[i]->p_parent = NULL;
+        /* We will protect ourselves directly in FIND_PARENT
+         * for FIND_PARENT */
     }
 
     vlc_mutex_unlock( &structure_lock );
@@ -1271,7 +1270,15 @@ static vlc_object_t * FindObject( vlc_object_t *p_this, int i_type, int i_mode )
         p_tmp = p_this->p_parent;
         if( p_tmp )
         {
-            if( p_tmp->i_object_type == i_type )
+            if( p_tmp->i_object_type == i_type
+                /* Special case. If the parent object
+                 * is being destroyed (i_refcount == 0)
+                 * don't return it. Note, p_tmp can't
+                 * be invalid memory, as we assert
+                 * that each child is detached from
+                 * its parent before parent vlc_object_t
+                 * structure is freed */
+                && vlc_internals( p_tmp )->i_refcount > 0 )
             {
                 vlc_object_yield( p_tmp );
                 return p_tmp;
@@ -1325,7 +1332,15 @@ static vlc_object_t * FindObjectName( vlc_object_t *p_this,
         if( p_tmp )
         {
             if( p_tmp->psz_object_name
-                && !strcmp( p_tmp->psz_object_name, psz_name ) )
+                && !strcmp( p_tmp->psz_object_name, psz_name )
+                /* Special case. If the parent object
+                 * is being destroyed (i_refcount == 0)
+                 * don't return it. Note, p_tmp can't
+                 * be invalid memory, as we assert
+                 * that each child is detached from
+                 * its parent before parent vlc_object_t
+                 * structure is freed */
+                && vlc_internals( p_tmp )->i_refcount > 0 )
             {
                 vlc_object_yield( p_tmp );
                 return p_tmp;
-- 
1.5.4.4.481.g5075

